unit CH341DLL; {Original von abacom}

interface

// 2004.05.28, 2004.10.20, 2005.01.08, 2005.03.25
//****************************************
//**  Copyright  (C)  W.ch  1999-2005   **
//**  Web:  http://www.winchiphead.com  **
//****************************************
//**  DLL for USB interface chip CH341  **
//**  Borland Delphi                    **
//****************************************
//
// CH341-DLL  V1.5
// support USB chip: CH341, CH341A
// USB => Parallel, I2C, SPI, RS232, ...
//
// bersetzung ABACOM-Ingnieurgesellschaft 2007 

uses SysUtils, WinTypes;

const
     FILE_DEVICE_UNKNOWN = $22;
     FILE_ANY_ACCESS = 0;
     METHOD_BUFFERED = 0;
     mCH341_PACKET_LENGTH = 32;
     mCH341_PKT_LEN_SHORT = 8 ;

     mCH341_MAX_NUMBER =	16;
     mMAX_BUFFER_LENGTH	=$1000;
     mDEFAULT_BUFFER_LEN=$0400;

     mCH341_ENDP_INTER_UP=$81 ;
     mCH341_ENDP_INTER_DOWN=$01;
     mCH341_ENDP_DATA_UP=$82  ;
     mCH341_ENDP_DATA_DOWN=$02 ;

     mPipeDeviceCtrl=$00000004;
     mPipeInterUp=$00000005  ;
     mPipeDataUp =$00000006  ;
     mPipeDataDown=$00000007 ;

     mFuncNoOperation =$00000000;
     mFuncGetVersion  =$00000001;
     mFuncGetConfig   =$00000002;
     mFuncSetTimeout  =$00000009;
     mFuncSetExclusive=$0000000b;
     mFuncResetDevice =$0000000c;
     mFuncResetPipe   =$0000000d;
     mFuncAbortPipe   =$0000000e;

     mFuncSetParaMode		=$0000000f;
     mFuncReadData0   		=$00000010;
     mFuncReadData1   		=$00000011;
     mFuncWriteData0  		=$00000012;
     mFuncWriteData1  		=$00000013;
     mFuncWriteRead   		=$00000014;

     mUSB_CLR_FEATURE		=$01;
     mUSB_SET_FEATURE		=$03;
     mUSB_GET_STATUS		=$00;
     mUSB_SET_ADDRESS		=$05;
     mUSB_GET_DESCR		=$06;
     mUSB_SET_DESCR		=$07;
     mUSB_GET_CONFIG		=$08;
     mUSB_SET_CONFIG		=$09;
     mUSB_GET_INTERF		=$0a;
     mUSB_SET_INTERF		=$0b;
     mUSB_SYNC_FRAME		=$0c;

     mCH341_VENDOR_READ		=$C0;
     mCH341_VENDOR_WRITE	=$40;

     mCH341_PARA_INIT		=$B1;
     mCH341_I2C_STATUS		=$52;
     mCH341_I2C_COMMAND		=$53;

     mCH341_PARA_CMD_R0		=$AC;
     mCH341_PARA_CMD_R1		=$AD;
     mCH341_PARA_CMD_W0		=$A6;
     mCH341_PARA_CMD_W1		=$A7;
     mCH341_PARA_CMD_STS	=$A0;

     mCH341A_CMD_SET_OUTPUT	=$A1;
     mCH341A_CMD_IO_ADDR	=$A2;
     mCH341A_CMD_SPI_STREAM	=$A8;
     mCH341A_CMD_SIO_STREAM	=$A9;
     mCH341A_CMD_I2C_STREAM	=$AA;
     mCH341A_CMD_UIO_STREAM	=$AB;

     mCH341A_BUF_CLEAR		=$B2;
     mCH341A_I2C_CMD_X		=$54;
     mCH341A_DELAY_MS		=$5E;
     mCH341A_GET_VER		=$5F;

     mCH341_EPP_IO_MAX	 =	( mCH341_PACKET_LENGTH - 1 );
     mCH341A_EPP_IO_MAX	 =      $FF;

     mCH341A_CMD_IO_ADDR_W	=$00;
     mCH341A_CMD_IO_ADDR_R	=$80;

     mCH341A_CMD_I2C_STM_STA	=$74;
     mCH341A_CMD_I2C_STM_STO	=$75;
     mCH341A_CMD_I2C_STM_OUT	=$80;
     mCH341A_CMD_I2C_STM_IN	=$C0;

     mCH341A_CMD_I2C_STM_SET	=$60;
     mCH341A_CMD_I2C_STM_US	=$40;
     mCH341A_CMD_I2C_STM_MS	=$50;
     mCH341A_CMD_I2C_STM_DLY	=$0F;
     mCH341A_CMD_I2C_STM_END	=$00;

     mCH341A_CMD_UIO_STM_IN	=$00;
     mCH341A_CMD_UIO_STM_DIR	=$40;
     mCH341A_CMD_UIO_STM_OUT	=$80;
     mCH341A_CMD_UIO_STM_US	=$C0;
     mCH341A_CMD_UIO_STM_END	=$20;

     mCH341_PARA_MODE_EPP	=$00;
     mCH341_PARA_MODE_EPP17	=$00;
     mCH341_PARA_MODE_EPP19	=$01;
     mCH341_PARA_MODE_MEM	=$02;


     mStateBitERR  		=$00000100;
     mStateBitPEMP 		=$00000200;
     mStateBitINT  		=$00000400;
     mStateBitSLCT 		=$00000800;
     mStateBitSDA  		=$00800000;

type
   PVOID = Pointer;
   PCardinal=^Cardinal;
   PULONG=pcardinal;

   mUspValue=record
             mUspValueLow : Byte;
             mUspValueHigh : Byte;
             end;

   mUspIndex=record
             mUspIndexLow : Byte;
             mUspIndexHigh  : Byte;
             end;

   USB_SETUP_PKT=record
                 mUspReqType : Byte;
                 mUspRequest : Byte;
                 mUspValue : mUspValue;
                 mUspIndex : mUspIndex;
                 mLength : Integer;
                 end ;

   WIN32_COMMAND=record
                 mFunction : cardinal;
                 mLength : cardinal;
                 mBuffer:array[0..(mCH341_PACKET_LENGTH-1)] of Byte;
                 end ;

   mPWIN32_COMMAND=^WIN32_COMMAND;

   mPCH341_INT_ROUTINE=Procedure (iStatus:cardinal);stdcall;

var
   mUSB_SETUP_PKT: USB_SETUP_PKT;
   mWIN32_COMMAND: WIN32_COMMAND;

// **************
// USB Funktionen
// **************
type    mPCH341_NOTIFY_ROUTINE =ProceDure (iEventStatus:cardinal); stdcall;
const	CH341_DEVICE_ARRIVAL =		3 ;
	CH341_DEVICE_REMOVE_PEND =	1 ;
	CH341_DEVICE_REMOVE    =	0 ;
Function CH341SetDeviceNotify(iIndex:cardinal; iDeviceID: PCHAR; iNotifyRoutine:mPCH341_NOTIFY_ROUTINE):boolean;Stdcall; external 'CH341DLL.DLL';


// *********************
// Allgemeine Funktionen
// *********************

// Alle Funktionen addressieren einen CH341A am USB mit dem Parameter 'iIndex':
// iIndex=0 => 1. CH341 am USB
// iIndex=1 => 2. CH341 am USB
// iIndex=2 => 3. ...

// CH341OpenDevice: Verbindung mit CH341A herstellen
Function CH341OpenDevice(iIndex: cardinal): integer; Stdcall; external 'CH341DLL.DLL' ;
// CH341CloseDevice: Verbindung mit CH341A beenden
procedure CH341CloseDevice(iIndex: cardinal); Stdcall; external 'CH341DLL.DLL';
// CH341ResetDevice: CH341A zurcksetzen auf Einschaltzustand
Function CH341ResetDevice(iIndex: cardinal): boolean; Stdcall; external 'CH341DLL.DLL';
// CH341SetExclusive: Weiteres ffnen verhindern.
// iExclusive: 0=shared ; mehrfaches ffen mglich
//             1=exclusive; kein weiteres ffnen mglich
Function CH341SetExclusive(iIndex:cardinal; iExclusive:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';

// **********************
// Informationen abfragen
// **********************
Function CH341GetVersion: cardinal; Stdcall; external 'CH341DLL.DLL';
Function CH341DriverCommand(iIndex: cardinal; ioCommand: mPWIN32_COMMAND):cardinal; Stdcall; external 'CH341DLL.DLL';
Function CH341GetDrvVersion: cardinal; Stdcall; external 'CH341DLL.DLL';
Function CH341GetDeviceDescr(iIndex: cardinal; oBuffer: PVOID; ioLength: PULONG): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341GetConfigDescr(iIndex: cardinal; oBuffer: PVOID; ioLength: PULONG): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341GetDeviceName(iIndex:cardinal):PVOID;Stdcall; external 'CH341DLL.DLL';
Function CH341GetVerIC(iIndex:cardinal):cardinal;Stdcall; external 'CH341DLL.DLL';

// **********
// Interrupts
// **********
Function CH341SetIntRoutine(iIndex: cardinal; iIntRoutine: mPCH341_INT_ROUTINE): boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341ReadInter(iIndex: cardinal; iStatus:PULONG): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341AbortInter(iIndex: cardinal): boolean; Stdcall; external 'CH341DLL.DLL';

// Buffer (USB?) RS-232?
Function CH341SetBufUpload(iIndex:cardinal; iEnableOrClear:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341QueryBufUpload(iIndex:cardinal):integer;Stdcall; external 'CH341DLL.DLL';
Function CH341SetBufDownload(iIndex:cardinal; iEnableOrClear:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341QueryBufDownload(iIndex:cardinal ):integer;Stdcall; external 'CH341DLL.DLL';
// (USB?) RS-232?
Function CH341ResetInter(iIndex:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341ResetRead(iIndex:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341ResetWrite(iIndex:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';

// USB ? RS-232?
//Delay;  iDelay: Unit 1ms
Function CH341SetDelaymS(iIndex:cardinal; iDelay:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
//Timeout;  iWriteTimeout: Unit 1ms
Function CH341SetTimeout(iIndex:cardinal; iWriteTimeout:cardinal; iReadTimeout:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341ReadData(iIndex:cardinal;	oBuffer:PVOID; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341WriteData(iIndex:cardinal; iBuffer:PVOID;	ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341FlushBuffer(iIndex:cardinal):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341WriteRead(iIndex:cardinal; iWriteLength:cardinal;	iWriteBuffer:PVOID; iReadStep:cardinal;	iReadTimes:cardinal; oReadLength:PULONG; oReadBuffer:PVOID ):boolean;Stdcall; external 'CH341DLL.DLL' ;

// ************************************
// Parallele Datenbertragung (EPP/MEM)
// ************************************
// iMode: 0 = EPP/EPP V1.7 ; 1 = EPP V1.9, 2 = MEM; 256 = keep current
Function CH341InitParallel(iIndex: cardinal; iMode :cardinal): boolean; Stdcall; external 'CH341DLL.DLL';
// iMode: 0 = EPP/EPP V1.7 ; 1 = EPP V1.9, 2 = MEM
Function CH341SetParaMode(iIndex: cardinal; iMode: cardinal): boolean; Stdcall; external 'CH341DLL.DLL';
// Read & Write (MEM und EPP)
Function CH341ReadData0(iIndex: cardinal; oBuffer: PVOID; ioLength: PULONG ): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341ReadData1(iIndex: cardinal; oBuffer: PVOID; ioLength: PULONG ): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341AbortRead(iIndex: cardinal): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341WriteData0(iIndex:cardinal; iBuffer: PVOID; ioLength: PULONG ): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341WriteData1(iIndex:cardinal; iBuffer: PVOID; ioLength: PULONG ): boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341AbortWrite(iIndex:cardinal): boolean; Stdcall; external 'CH341DLL.DLL';

// EPP Read/Write
Function CH341EppReadData(iIndex:cardinal; oBuffer:PVOID; ioLength:PULONG ):boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341EppReadAddr(iIndex:cardinal; oBuffer:pvoid; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341EppWriteData(iIndex:cardinal; iBuffer:pvoid; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341EppWriteAddr(iIndex:cardinal; iBuffer:PVOID; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341EppSetAddr(iIndex:cardinal; iAddr:byte ):boolean;Stdcall; external 'CH341DLL.DLL';

// MEM Read/Write
Function CH341MemReadAddr0(iIndex:cardinal; oBuffer:pvoid; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341MemReadAddr1(iIndex:cardinal; oBuffer:pvoid; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
function CH341MemWriteAddr0(iIndex:cardinal; iBuffer:pvoid; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341MemWriteAddr1(iIndex:cardinal; iBuffer:pvoid; ioLength:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';

// *******************************************
// Serielle Datenbertragung (I2C, RS232, SPI)
// *******************************************

// CH341SetStream() konfiguriert I2C und SPI
// Bit 1-0: I2C speed 00= low speed /20KHz
//                    01= standard /100KHz
//                    10= fast /400KHz
//                    11= high speed /750KHz

// Bit 2: SPI - Modus
//                      0= Standard SPI (D3=Takt, D5=ser. Ausgang, D7 ser. Eingang
//                      1= SPI mit doppelten Datenleitungen (D3=Takt, D5,D4=ser. Ausgnge, D7,D6 ser. Eingnge)

// Bit 7: SPI           0= LSB first
//                      1= MSB first

// andere Bits          0 (reserviert)
Function CH341SetStream(iIndex:cardinal; iMode:cardinal):boolean;Stdcall; external 'CH341DLL.DLL';

// *********************************
// SPI - Serial Peripheral Interface
// *********************************
//
// CH341A ist immer MASTER
//
// CH341BitStreamSPI()
// Serieller Datenaustausch mit (nicht SPI-konformen) seriellen ICs (z.B. LTC1257)
// Gleichzeitiges Lesen und Schreiben ber die Bytes im ioBuffer-Byte-Array:

// Bit7: D7 <- Eingang (Lesen von Peripherie-IC)
// Bit6: D6 <- Eingang (Lesen von Peripherie-IC)
// Bit5: D5 -> Ausgang (zum Peripherie-IC)
// Bit4: D4 -> Ausgang (zum Peripherie-IC)
// Bit3: D3 -> Ausgang (SCLK; serieller Takt wird von CH341A automatisch erzeugt);
// Bit2: D2 -> Ausgang (zum Peripherie-IC)
// Bit1: D1 -> Ausgang (zum Peripherie-IC)
// Bit0: D0 -> Ausgang (zum Peripherie-IC)

// Vor Aufruf der Funktion das gewnschte Signalmuster (Pattern) in die Bits D0,D1,D2,D4,D5 einsetzen
// Bei Aufruf der Funktion werden die Daten SERIELL ausgegeben (D0..D5) UND eingelesen (D6,D7)
// Nach dem Aufruf enthalten die Bits D6 und D7 die gelesenen Daten.
// D3 stellt automatisch ein Taksignal bereit. (Positiver Takimpuls fr jedes Byte im IO-Buffer)
// Vor dem Aufruf aufruf mssen die Ausgnge aktiviert werden mit Set_D5_D0(iIndex, $3F) //D0...D5 als Ausgang
// iLength ist die Anzahl der Byte die gelesen/geschrieben werden.
Function CH341BitStreamSPI(iIndex:cardinal; iLength:cardinal; ioBuffer:PVOID):boolean;Stdcall; external 'CH341DLL.DLL';


// CH341StreamSPI4()
// Standard-SPI (Mode 0) mit zwei unidirektionalen Datenleitungen, Clock und Chip-Select.
// z.B. Atmel, etc.
// D7 <- Eingang (Lesen von Peripherie-IC; MISO)
// D5 -> Ausgang (Schreiben zum Peripherie-IC; MOSI)
// D3 -> Ausgang (SCLK; serieller Takt wird von CH341A automatisch erzeugt);
// D2 -> Ausgang (zum Peripherie-IC; Chip Select 2)
// D1 -> Ausgang (zum Peripherie-IC; Chip Select 1)
// D0 -> Ausgang (zum Peripherie-IC; Chip Select 0)
Function CH341StreamSPI4(iIndex:cardinal; iChipSelect:cardinal; iLength:cardinal; ioBuffer:PVOID ):boolean;Stdcall; external 'CH341DLL.DLL';

// CH341StreamSPI3()
// SPI mit nur EINER bi-direktionalen(!) Datenleitung, Clock und Chip-Select
// z.B. bei Maxim-Chips (DS1626); (Schreiben mit steigendem Takt; Lesen mit fallendem Takt)
Function CH341StreamSPI3(iIndex:cardinal; iChipSelect:cardinal;	iLength:cardinal; ioBuffer:PVOID):boolean;Stdcall; external 'CH341DLL.DLL';

// CH341StreamSPI5()
// SPI mit 4 Datenleitungen (2 Paare unidirektional); Beispiel nicht bekannt
Function CH341StreamSPI5(iIndex:cardinal; iChipSelect:cardinal; iLength:cardinal; ioBuffer:PVOID; ioBuffer2:PVOID ):boolean;Stdcall; external 'CH341DLL.DLL';

// ***********
// *** I2C ***
// ***********
//
// CH341A ist immer MASTER
//
// CH341StreamI2C(): Universelles Lesen und/oder Schreiben ber I2C
// Nur Lesen: iWriteLength = 0;
// Nur Schreiben: iReadLength = 0;
Function CH341StreamI2C(iIndex:cardinal; iWriteLength:cardinal; iWriteBuffer:pvoid; iReadLength:cardinal; oReadBuffer:pvoid ):boolean;stdcall; external 'CH341DLL.DLL';

// Lesen und Schreiben von IC mit interner Speicheradresse
// iDevice: 0..127 = I2C-Gerteadresse (device address)
// iAddr: Speicheradresse; Dies ist NICHT die I2C-Gerteadresse!
Function CH341ReadI2C(iIndex:cardinal; iDevice: byte; iAddr: byte; oByte: pbyte ):boolean; Stdcall; external 'CH341DLL.DLL';
Function CH341WriteI2C(iIndex:cardinal; iDevice :byte; iAddr: byte; iByte: byte ):boolean; Stdcall; external 'CH341DLL.DLL';
// I2C-EEPROM lesen und schreiben
type EEPROM_TYPE =(ID_24C01,ID_24C02,ID_24C04,ID_24C08,ID_24C16,ID_24C32,ID_24C64,ID_24C128,ID_24C256,ID_24C512,ID_24C1024,ID_24C2048,ID_24C4096);
Function CH341ReadEEPROM(iIndex:cardinal; iEepromID:EEPROM_TYPE; iAddr:cardinal; iLength:cardinal; oBuffer:Pbytearray ):boolean;stdcall; external 'CH341DLL.DLL';
Function CH341WriteEEPROM(iIndex:cardinal; iEepromID:EEPROM_TYPE; iAddr:cardinal; iLength:cardinal; iBuffer:pbytearray ):boolean;stdcall; external 'CH341DLL.DLL';

// ***************************************
// Direktes Setzen der I/O Pins des CH341A
// ***************************************

// iEnable:       Sperrt bzw. erlaubt die Umschaltung der Datenrichtung
//                Bit gesetzt = nderung zugelassen
//                Bit gelscht = nderungen werden ignoriert
// iSetDirOut:    Umschaltung der Datenrichtung: Eingang oder Ausgang
//                Bit gesetzt  = Pin arbeitet als Ausgang (Vorsicht!)
//                Bit gelscht = Pin arbeitet als ist Eingang
// iSetDataOut:   Ausgnge schalten
//                Bit gesetzt  = Ausgang HIGH schalten
//                Bit gelscht = Ausgang LOW schalten

// Zuordnung der Bit zu den Pins des CH341A:

// Bidirektionale Pins (programmierbarer Eingang oder Ausgang):
// Bit 0-7  = D0-D7          pin 15-22
// Bit 8    = ERR#           pin 5
// Bit 9    = PEMP           pin 6
// Bit 10   = ACK            pin 7
// Bit 11   = SLCT           pin 8
// Bit 12   = unused         -
// Bit 13   = BUSY/WAIT#     pin 27
// Bit 14   = AUTOFD#/DATAS# pin 4
// Bit 15   = SLCTIN#/ADDRS# pin 3

// quasi-bidirektional (Kombination aus Eingang und Open-Collector-Ausgang) :
// Bit 23   = SDA            pin 23 (bei Funktionen GetInput/GetStatus)
// Bit 16   = SDA            pin 23 (bei Funktion SetOutput)

// Uni-direktionale Pins (immer Ausgang):
// Bit 16   = RESET#         pin 26
// Bit 17   = WRITE#         pin 25
// Bit 18   = SCL pin        pin 24

// Anmerkung: Der Aufruf anderer API-Funktionen (z.B. RS232, SPI, ...
// kann den Zustand der Leitungen ndern! Das Aufeinandertreffen von zwei Ausgngen
// kann zur Beschdigung des Chips fhren. Rufen Sie daher stets nur Funktionen auf
// deren Funktionsweise Ihnen bekannt ist und die zur externen Beschaltung des Chips passen.

//Ausgnge setzen
Function CH341SetOutput(iIndex:cardinal; iEnable:cardinal; iSetDirOut:cardinal;iSetDataOut:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341Set_D5_D0(iIndex:cardinal; iSetDirOut:cardinal; iSetDataOut:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';
// Eingnge lesen
Function CH341GetInput(iIndex:cardinal; iStatus:PULONG ):boolean;Stdcall; external 'CH341DLL.DLL';
Function CH341GetStatus(iIndex:cardinal; iStatus: PULONG ): boolean; Stdcall; external 'CH341DLL.DLL';

// ******
// RS-232
// ******

// iParityMode:
// 0 = NONE
// 1 = ODD
// 2 = EVEN
// 3 = MARK
// 4 = SPACE
// iBaudRate:
// 50, 75, 100, 110, 134.5, 150, 300, 600, 900, 1200, 1800, 2400, 3600, 4800,
// 9600, 14400, 19200, 28800, 33600, 38400, 56000, 57600, 76800, 115200, 128000,
// 153600, 230400, 460800, 921600, 1500000, 2000000
Function CH341SetupSerial(iIndex:cardinal; iParityMode:cardinal; iBaudRate:cardinal ):boolean;Stdcall; external 'CH341DLL.DLL';


implementation

end.
